@@ -12,16 +12,15 @@ from typing import (
     Tuple,
 )
 
+import numpy as np
 from instancelib import Instance
+from instancelib.utils.func import union, value_map
 
 from ..activelearning.base import ActiveLearner
 from ..environment import AbstractEnvironment
 from ..environment.base import AbstractEnvironment
 from ..typehints import DT, IT, KT, LT, RT, VT
 from .base import ActiveLearner
-import numpy as np
-
-from instancelib.utils.func import value_map
 
 
 class AbstractStatistics(ABC, Generic[KT, LT]):
@@ -64,11 +63,13 @@ class AbstractStatistics(ABC, Generic[KT, LT]):
         raise NotImplementedError
 
     def annotations_since_last(self, label: LT) -> int:
-        last_pos_it = max(
-            [it for it, count in enumerate(self.label_per_round(label)) if count > 0]
-        )
-        annotated_at = np.array(self.annotations_per_round).cumsum()[last_pos_it]
-        return self.current_annotated - annotated_at
+        label_enumerator = list(enumerate(self.label_per_round(label)))
+        pos_iterations = [it for it, count in label_enumerator if count > 0]
+        if pos_iterations:
+            last_pos_it = max(pos_iterations)
+            annotated_at = np.array(self.annotations_per_round).cumsum()[last_pos_it]
+            return self.current_annotated - annotated_at
+        return 0
 
 
 class StatsMixin(ABC, Generic[KT, LT]):
@@ -209,10 +210,12 @@ class AnnotationStatistics(AbstractStatistics[KT, LT]):
 class AnnotationStatisticsSlim(AbstractStatistics[KT, LT]):
     labelwise: List[Mapping[LT, int]]
     per_round: List[Mapping[LT, int]]
+    previous_round: FrozenSet[KT]
     annotated_per_round: List[int]
     annotated: List[int]
     unlabeled: List[int]
     dataset: List[int]
+    previous_rounds: FrozenSet[KT]
 
     def __init__(self) -> None:
         self.labelwise = list()
@@ -249,7 +252,7 @@ class AnnotationStatisticsSlim(AbstractStatistics[KT, LT]):
         self.unlabeled.append(len(unlabeled))
         self.per_round.append(current_round_new_counts)
         self.dataset.append(len(learner.env.dataset))
-        self.previous_round = current_round
+        self.previous_round = annotated
 
     @property
     def dataset_size(self) -> int:
