@@ -307,11 +307,11 @@ def cmhs(
     alpha: float = 0.05,
 ) -> Tuple[Mapping[str, AbstractEstimator], Mapping[str, AbstractStopCriterion[LT]]]:
     cmh_certs = {
-        f"CMH_CERT_{target}": CMH_CertificationRule(pos_label, target, alpha)
+        f"CMH_Hybrid_{target}": CMH_CertificationRule(pos_label, target, alpha)
         for target in recall_targets
     }
     cmh_heurs = {
-        f"CMH_HEUR_{target}": CMH_HeuristicMethodRuleTwoPhase(pos_label, target, alpha)
+        f"CMH_Standoff_{target}": CMH_HeuristicMethodRuleTwoPhase(pos_label, target, alpha)
         for target in recall_targets
     }
     return dict(), {**cmh_certs, **cmh_heurs}
@@ -411,7 +411,7 @@ EXPERIMENT_REPOSITORY: Mapping[ExperimentCombination, TarExperimentParameters] =
         ALConfiguration.AUTOTAR,
         None,
         SeededRandomInitializer.builder(5),
-        (StopBuilderConfiguration.AUTOTAR,),
+        (StopBuilderConfiguration.AUTOTAR,StopBuilderConfiguration.QUANT,),
         10,
         10,
         10,
