@@ -1,5 +1,6 @@
-from typing import Any, Mapping
+from typing import Any, Mapping, Tuple
 from ..module import ModuleCatalog as Cat
+from lenses import lens
 
 
 def add_identifier(config: Mapping[str, Any], identifier: str) -> Mapping[str, Any]:
@@ -9,8 +10,8 @@ def add_identifier(config: Mapping[str, Any], identifier: str) -> Mapping[str, A
 
 
 def set_batch_size(config: Mapping[str, Any], batch_size: int) -> Mapping[str, Any]:
-    id_dict = {"batch_size": batch_size}
-    new_dict = {**config, **id_dict}
+    l = lens.Item("batch_size").set(("batch_size", batch_size))
+    new_dict = l(config)
     return new_dict
 
 
@@ -106,10 +107,14 @@ DOUBLEBALANCER = {"type": Cat.BL.Type.DOUBLE, "config": {}}
 IDENTITYBALANCER = {"type": Cat.BL.Type.IDENTITY, "config": {}}
 
 
-def btar(machinelearning: Mapping[str, Any], batch_size: int = 10):
+def btar(
+    machinelearning: Mapping[str, Any],
+    batch_size: int = 10,
+    method: Cat.AL.CustomMethods = Cat.AL.CustomMethods.BINARYTAR,
+):
     config = {
         "paradigm": Cat.AL.Paradigm.CUSTOM,
-        "method": Cat.AL.CustomMethods.BINARYTAR,
+        "method": method,
         "machinelearning": machinelearning,
         "batch_size": batch_size,
     }
@@ -123,8 +128,8 @@ def autotar(
         "paradigm": Cat.AL.Paradigm.CUSTOM,
         "method": Cat.AL.CustomMethods.AUTOTAR,
         "machinelearning": machinelearning,
-        "k_sample": 100,
-        "batch_size": 20,
+        "k_sample": k_sample,
+        "batch_size": batch_size,
     }
     return config
 
@@ -145,45 +150,45 @@ def tar_classifier(
     return config
 
 
-al_il_config_nb = btar(
-    machinelearning=tar_classifier(
-        sklearn_model=Cat.ML.SklearnModel.NAIVE_BAYES,
-        model_configuration={"alpha": 3.822},
-        feature_extraction=tf_idf_autotar,
-        balancer=DOUBLEBALANCER,
-    ),
-    batch_size=10,
+NB = (Cat.ML.SklearnModel.NAIVE_BAYES, {"alpha": 3.822})
+LR = (
+    Cat.ML.SklearnModel.LOGISTIC,
+    {
+        "solver": "lbfgs",
+        "C": 1.0,
+        "max_iter": 10000,
+    },
 )
-al_il_config_lr = btar(
-    machinelearning=tar_classifier(
-        sklearn_model=Cat.ML.SklearnModel.LOGISTIC,
-        model_configuration=dict(),
-        feature_extraction=tf_idf_autotar,
-        balancer=DOUBLEBALANCER,
-    ),
-    batch_size=10,
-)
-al_il_config_lgbm = btar(
-    machinelearning=tar_classifier(
-        sklearn_model=Cat.ML.SklearnModel.LGBM,
-        model_configuration=dict(),
-        feature_extraction=tf_idf_autotar,
-        balancer=DOUBLEBALANCER,
-    ),
-    batch_size=10,
-)
-al_il_config_rf = btar(
-    machinelearning=tar_classifier(
-        sklearn_model=Cat.ML.SklearnModel.RANDOM_FOREST,
-        model_configuration={
-            "n_estimators": 100,
-            "max_features": 10,
-        },
-        feature_extraction=tf_idf_autotar,
-        balancer=DOUBLEBALANCER,
-    ),
-    batch_size=10,
+LGBM = (Cat.ML.SklearnModel.LGBM, {"n_jobs": 1})
+RF = (
+    Cat.ML.SklearnModel.RANDOM_FOREST,
+    {
+        "n_estimators": 100,
+        "max_features": 10,
+    },
 )
+
+
+def sk_btar(
+    model: Tuple[Cat.ML.SklearnModel, Mapping[str, Any]],
+    batch_size: int = 10,
+    fe: Mapping[str, Any] = tf_idf_autotar,
+    balancer: Mapping[str, Any] = DOUBLEBALANCER,
+    method: Cat.AL.CustomMethods = Cat.AL.CustomMethods.BINARYTAR,
+) -> Mapping[str, Any]:
+    model_type, model_config = model
+    return btar(
+        machinelearning=tar_classifier(
+            sklearn_model=model_type,
+            model_configuration=model_config,
+            feature_extraction=fe,
+            balancer=balancer,
+        ),
+        batch_size=batch_size,
+        method=method,
+    )
+
+
 al_config_nb = {
     "paradigm": Cat.AL.Paradigm.LABEL_PROBABILITY_BASED,
     "query_type": Cat.AL.QueryType.LABELMAXIMIZER,
@@ -399,16 +404,139 @@ rasch_nblrrflgbmrand = {
     ],
 }
 
-ilrasch_nblrrflgbmrand = {
-    "paradigm": Cat.AL.Paradigm.ESTIMATOR,
-    "learners": [
-        set_batch_size(add_identifier(al_il_config_nb, "NaiveBayes"), 200),
-        set_batch_size(add_identifier(al_il_config_rf, "RandomForest"), 200),
-        set_batch_size(add_identifier(al_il_config_lr, "LogisticRegression"), 200),
-        set_batch_size(add_identifier(al_il_config_lgbm, "LGBM"), 200),
-        set_batch_size(add_identifier(al_config_random, "Random"), 200),
-    ],
-}
+
+def chao_ensemble(
+    batch_size: int,
+    tf_idf: Mapping[str, Any] = tf_idf_autotar,
+    method=Cat.AL.CustomMethods.BINARYTAR,
+) -> Mapping[str, Any]:
+    return {
+        "paradigm": Cat.AL.Paradigm.ESTIMATOR,
+        "learners": [
+            add_identifier(
+                sk_btar(NB, batch_size, tf_idf, DOUBLEBALANCER, method=method),
+                "NaiveBayes",
+            ),
+            add_identifier(
+                sk_btar(RF, batch_size, tf_idf, DOUBLEBALANCER, method=method),
+                "RandomForest",
+            ),
+            add_identifier(
+                sk_btar(LGBM, batch_size, tf_idf, DOUBLEBALANCER, method=method), "LGBM"
+            ),
+            add_identifier(
+                sk_btar(LR, batch_size, tf_idf, DOUBLEBALANCER, method=method),
+                "LogisticRegression",
+            ),
+            set_batch_size(add_identifier(al_config_random, "Random"), batch_size),
+        ],
+    }
+
+
+def chao_ensemble2(
+    batch_size: int,
+    tf_idf: Mapping[str, Any] = tf_idf_autotar,
+    method=Cat.AL.CustomMethods.BINARYTAR,
+) -> Mapping[str, Any]:
+    return {
+        "paradigm": Cat.AL.Paradigm.ESTIMATOR,
+        "learners": [
+            add_identifier(
+                sk_btar(NB, batch_size, tf_idf, DOUBLEBALANCER, method=method),
+                "NaiveBayes",
+            ),
+            add_identifier(
+                sk_btar(RF, batch_size, tf_idf, DOUBLEBALANCER, method=method),
+                "RandomForest",
+            ),
+            add_identifier(
+                sk_btar(LGBM, batch_size, tf_idf, DOUBLEBALANCER, method=method), "LGBM"
+            ),
+            add_identifier(
+                sk_btar(LR, batch_size, tf_idf, DOUBLEBALANCER, method=method),
+                "LogisticRegression",
+            ),
+            add_identifier(
+                sk_btar(LR, batch_size, tf_idf, DOUBLEBALANCER, method=method),
+                "LogisticRegression2",
+            ),
+        ],
+    }
+
+
+def chao_ensemble_same(
+    batch_size: int,
+    tf_idf: Mapping[str, Any] = tf_idf_autotar,
+    clf: Mapping[str, Any] = LR,
+    name: str = "CLF",
+    method=Cat.AL.CustomMethods.BINARYTAR,
+) -> Mapping[str, Any]:
+    return {
+        "paradigm": Cat.AL.Paradigm.ESTIMATOR,
+        "learners": [
+            add_identifier(
+                sk_btar(clf, batch_size, tf_idf, DOUBLEBALANCER, method=method),
+                f"{name}_{i}",
+            )
+            for i in range(1, 6)
+        ],
+    }
+
+
+def chao_ensemble_same_random(
+    batch_size: int,
+    tf_idf: Mapping[str, Any] = tf_idf_autotar,
+    method=Cat.AL.CustomMethods.BINARYTAR,
+) -> Mapping[str, Any]:
+    return {
+        "paradigm": Cat.AL.Paradigm.ESTIMATOR,
+        "learners": [
+            set_batch_size(add_identifier(al_config_random, f"Random_{i}"), batch_size)
+            for i in range(1, 6)
+        ],
+    }
+
+
+def chao_ensemble_prior(
+    batch_size: int,
+    tf_idf: Mapping[str, Any] = tf_idf_autotar,
+    method=Cat.AL.CustomMethods.BINARYTAR,
+    nneg: int = 10,
+    nirel: int = 10,
+) -> Mapping[str, Any]:
+    return {
+        "paradigm": Cat.AL.Paradigm.CUSTOM,
+        "method": Cat.AL.CustomMethods.PRIORAUTOTAR,
+        "tarmethod": autotar(tar_classifier(LR[0], LR[1], tf_idf), 100, 20),
+        "ensemble": chao_ensemble(batch_size, tf_idf, method),
+        "nneg": nneg,
+        "nirel": nirel,
+    }
+
+
+def targetmethod(tf_idf: Mapping[str, Any] = tf_idf_autotar) -> Mapping[str, Any]:
+    return {
+        "paradigm": Cat.AL.Paradigm.CUSTOM,
+        "method": Cat.AL.CustomMethods.TARGET,
+        "tarmethod": autotar(tar_classifier(LR[0], LR[1], tf_idf), 100, 20),
+    }
+
+
+def cmhmethod(
+    tf_idf: Mapping[str, Any] = tf_idf_autotar,
+    target_recall=0.95,
+    alpha=0.05,
+    **kwargs: Any,
+) -> Mapping[str, Any]:
+    return {
+        "paradigm": Cat.AL.Paradigm.CUSTOM,
+        "method": Cat.AL.CustomMethods.CMH,
+        "tarmethod": autotar(tar_classifier(LR[0], LR[1], tf_idf), 100, 20),
+        "target_recall": target_recall,
+        "alpha": alpha,
+        **kwargs,
+    }
+
 
 rasch_lr = {
     "paradigm": Cat.AL.Paradigm.ESTIMATOR,
@@ -477,7 +605,7 @@ al_config_est5 = {
 }
 autotars = {
     "NaiveBayes": (Cat.ML.SklearnModel.NAIVE_BAYES, {"alpha": 3.822}, 100, 20),
-    "Logistic": (
+    "LogisticRegression": (
         Cat.ML.SklearnModel.LOGISTIC,
         {
             "solver": "lbfgs",
@@ -487,7 +615,7 @@ autotars = {
         100,
         20,
     ),
-    "RF": (
+    "RandomForest": (
         Cat.ML.SklearnModel.RANDOM_FOREST,
         {
             "n_estimators": 100,
